/** @file
===========================================================================
MODULE          : CStatusInterface

DESCRIPTIONS    : 

REVISION HISTORY: Version 1.0

COPYRIGHT       : (c)CSIR, Meraka Institute 2007 all rights reserved

RESTRICTIONS	: The information/data/code contained within this file is 
				  the property of CSIR, Meraka Institute and has been 
				  classified as PUBLIC.
===========================================================================
*/
#pragma once 

#include <DirectShow/Common/StatusInterface.h>
#include <Windows.h>

/// ***************************************************************
///  Author: RGlobisch	
///	 CStatusInterface   version:  1.0     date: 01/09/2008
///  -------------------------------------------------------------
///  This base class takes care of communicating the filter status with the main application.
///  It supports 
///  GetLastError (A method to obtain the last error of the filter), notification via Windows event
///  (in which case the the situation is handled in the main thread), and FriendlyID (This gets used to identify the filter in the main application layer).
///  -------------------------------------------------------------
class CStatusInterface : public IStatusInterface
{
public:
	/// Constructor
	CStatusInterface();
	/// Sets the error string and notifies the application using the media event sink if set
	STDMETHODIMP SetLastError( std::string sError, bool bNotifyApplication = false);
	/// Returns the last error that occurred in the filter
	STDMETHODIMP GetLastError( std::string& sError );
	/// Mutator for media event sink which enables notification via Windows message passing
	STDMETHODIMP SetMediaEventSink( IMediaEventSink* pEventSink );
	/// Mutator for friendly ID
	STDMETHODIMP SetFriendlyID( long lId );
	/// Accessor for friendly ID
	STDMETHODIMP GetFriendlyID( long& lId );

protected:
	/// This method notifies the application with the specified params
	void NotifyApplication(long lEventCode, LONG_PTR lEventParam1);

	/// String to store last error
	std::string m_sLastError;
	/// Event sink used for windows message passing notification
	IMediaEventSink* m_pMediaEventSink;
	/// Friendly id of the filter
	long m_lFriendlyId;
};