/** @file
===========================================================================
MODULE				: MultiIOBaseFilter

DESCRIPTIONS		: This base class takes care of doing most of the menial work when writing a transform filter that has multiple input and output pins.

REVISION HISTORY	: Version 1.0

COPYRIGHT			: (c)CSIR, Meraka Institute 2007 all rights reserved

RESTRICTIONS		: The information/data/code contained within this file is 
					  the property of CSIR, Meraka Institute and has been 
					  classified as PUBLIC.
===========================================================================
*/

#pragma once

#include <streams.h>

class CMultiIOInputPin : public CBaseInputPin
{
	friend class CMultiIOBaseFilter;
	friend class CMultiIOOutputPin;

public:
	CMultiIOInputPin(CMultiIOBaseFilter* pFilter, CCritSec* pLock, HRESULT* phr, LPCWSTR pName, int nIndex);
	virtual ~CMultiIOInputPin(void);

	// lifetime management for pins is normally delegated to the filter, but
	// we need to be able to create and delete them independently, so keep 
	// a separate refcount.
	STDMETHODIMP_(ULONG) NonDelegatingRelease()
	{
		return CUnknown::NonDelegatingRelease();
	}
	STDMETHODIMP_(ULONG) NonDelegatingAddRef()
	{
		return CUnknown::NonDelegatingAddRef();
	}

	// check that we can support this output type
	HRESULT CheckMediaType(const CMediaType* mtIn);

	// connection management -- used to maintain one free pin
	HRESULT BreakConnect();
	HRESULT CompleteConnect(IPin *pReceivePin);

	// do something with this media sample
	STDMETHODIMP Receive(IMediaSample *pSample);

	CMediaType& CurrentMediaType() { return m_mt; };

	// set the connection media type
	HRESULT SetMediaType(const CMediaType* mt);

	// Pass through calls downstream
	STDMETHODIMP EndOfStream();
	STDMETHODIMP BeginFlush();
	STDMETHODIMP EndFlush();
	STDMETHODIMP NewSegment(
		REFERENCE_TIME tStart,
		REFERENCE_TIME tStop,
		double dRate);
protected:
	CMultiIOBaseFilter* m_pFilter;
	int m_nIndex;
};
