/** @file
===========================================================================
MODULE          : DirectShowAudioPipeline

DESCRIPTIONS    : 

REVISION HISTORY: Version 1.0

COPYRIGHT       : (c)CSIR, Meraka Institute 2007 all rights reserved

RESTRICTIONS	: The information/data/code contained within this file is 
				  the property of CSIR, Meraka Institute and has been 
				  classified as PUBLIC.
===========================================================================
*/
#pragma once

#include <comdef.h>
_COM_SMARTPTR_TYPEDEF(IGraphBuilder, IID_IGraphBuilder);
_COM_SMARTPTR_TYPEDEF(IMediaControl, IID_IMediaControl);
_COM_SMARTPTR_TYPEDEF(IMediaEvent, IID_IMediaEvent);
_COM_SMARTPTR_TYPEDEF(IMediaEventEx, IID_IMediaEventEx);
_COM_SMARTPTR_TYPEDEF(IMediaEventSink, IID_IMediaEventSink);
_COM_SMARTPTR_TYPEDEF(IBaseFilter, IID_IBaseFilter);
_COM_SMARTPTR_TYPEDEF(ICaptureGraphBuilder2, IID_ICaptureGraphBuilder2);
_COM_SMARTPTR_TYPEDEF(ICreateDevEnum, IID_ICreateDevEnum);
_COM_SMARTPTR_TYPEDEF(IPin, IID_IPin);

_COM_SMARTPTR_TYPEDEF(IAMStreamConfig, IID_IAMStreamConfig);
//Sample Grabber Interface
#include <QEdit.h>
_COM_SMARTPTR_TYPEDEF(ISampleGrabber, IID_ISampleGrabber);
_COM_SMARTPTR_TYPEDEF(ISampleGrabberCB, IID_ISampleGrabberCB);

#include <DirectShow/Common/SettingsInterface.h>
_COM_SMARTPTR_TYPEDEF(ISettingsInterface, IID_ISettingsInterface);

#include <DirectShow/LiveIntegration/directshowdevicesource.h>
#include <Utils/AudioUtils/WavHeaderData.h>

class CDirectShowSampleGrabberCB;

// Create and manage the audio streaming pipeline in this class
class DirectShowAudioPipeline
{
public:
	DirectShowAudioPipeline(void);

	// Creates GraphManager
	bool InitDirectShow();

	std::string AudioDeviceName() const { return m_sAudioDeviceName; }
	void AudioDeviceName(std::string val) { m_sAudioDeviceName = val; }

	bool LoadPipeLine();

	bool Run();
	bool Pause();
	bool Stop();

public:
	~DirectShowAudioPipeline(void);

	WavHeaderData* MediaHeaderInfo() const { return m_pHeaderInfo; }

	void SetLive555AudioSource(CDirectShowDeviceSource* pSource);

private:
	HRESULT GetAudioMediaType(AM_MEDIA_TYPE** pMediaType);

	bool LoadFilters();
	bool ConnectFilters();

	// DirectShow graph components
	IGraphBuilderPtr m_pGraph;
	IMediaControlPtr m_pControl;	// Media control object
	
	// Filters
	IBaseFilterPtr m_pAudioCaptureSource;
	IBaseFilterPtr m_pAudioSampleGrabber;
	IBaseFilterPtr m_pAudioNullRenderer;
	IBaseFilterPtr m_pACMWrapper;

	// IAM Stream Config Interface to set compression parameters
	IAMStreamConfigPtr m_pIamStreamConfigInterface;

	// Sample grabber
	CDirectShowSampleGrabberCB* m_pAudioCB;
	ISampleGrabberPtr m_pSampleGrabberInterface;

	AM_MEDIA_TYPE* m_pMediaType;
	WavHeaderData* m_pHeaderInfo;
	
	//Window handle
	HWND m_hNotificationWnd;

	// Live DirectShow integration
	CDirectShowDeviceSource* m_pSource;
	// Audio input device name
	std::string m_sAudioDeviceName;
};
