// DirectShowAudioStreamingServer.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "DirectShowAudioStreamingServer.h"

// WIN32 Combobox
#include <commctrl.h>
#include <commdlg.h>
// This link is needed to get the comboboxes to display the correct size when they drop down
#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='*' publicKeyToken='6595b64144ccf1df' language='*'\"")

// Includes
#include "DirectShowAudioPipeline.h"
#include <Utils/DirectShowUtils/DirectShowHelper.h>

// LiveMedia includes
#include "liveMedia.hh"
#include "GroupsockHelper.hh"
#include "BasicUsageEnvironment.hh"

// DirectShow LiveMedia integration
#include <DirectShow/LiveIntegration/AudioDirectShowDeviceSource.h>
#include <DirectShow/LiveIntegration/PollingTaskScheduler.h>


// Global variables

HWND g_hWndMainWindow;
char g_pszResult[256];
bool g_bAudioDeviceSet = false;
std::string g_sLog;

// DirectShow vars
DirectShowAudioPipeline g_AudioPipeline;

// Live555 vars
UsageEnvironment* env;
static char g_WatchVariable;

// A structure to hold the state of the current session.
// It is used in the "afterPlaying()" function to clean up the session.
struct sessionState_t {
	FramedSource* source;
	RTPSink* sink;
	RTCPInstance* rtcpInstance;
	Groupsock* rtpGroupsock;
	Groupsock* rtcpGroupsock;
	RTSPServer* rtspServer;
} sessionState;

// Forward declarations
void AddTextToDisplay(std::string sText);
void UpdateDisplay(const char* szMessage);

// LiveMedia Forward Declarations
void play(); // forward
void afterPlaying(void* clientData); // forward

// Win32 Forward Declarations 
// Callback to handle out button clicks
BOOL CALLBACK MainDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam);
void PopulateComboBox(HWND parent, int ControlID, std::vector<std::string>& vData);

// LiveMedia Thread function
static DWORD WINAPI LiveRTSPServerThreadFunc(LPVOID pvParam)
{
	play();
	env->taskScheduler().doEventLoop(&g_WatchVariable);
	return 0; 
}


int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
	UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(lpCmdLine);

 	// TODO: Place code here.
	// Init COM
	CoInitializeEx(NULL, COINIT_APARTMENTTHREADED);
	
	//Create our main dialog
	g_hWndMainWindow = CreateDialog(GetModuleHandle(NULL), 
		MAKEINTRESOURCE(IDD_MAIN_DIALOG), 0, MainDlgProc);

	//Populate combobox with audio devices
	InitCommonControls();
	std::vector<std::string> vOptions;
	HRESULT hr = CDirectShowHelper::GetAudioInputs(vOptions);
	if (SUCCEEDED(hr))
	{
		PopulateComboBox(g_hWndMainWindow, IDC_AUDIO_DEVICE, vOptions);
	}
	else
	{
		// Display couldn't load audio input devices
		AddTextToDisplay("Couldn't load audio input devices");
	}

	//Show the main window
	ShowWindow(g_hWndMainWindow, nCmdShow);
	UpdateWindow(g_hWndMainWindow);

	// Main message loop:
	MSG msg;
	while (GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	CoUninitialize();
	return (int) msg.wParam;
}


//////////////////////////////////////////////////////////////////////////
// Win32 handlers

void PopulateComboBox(HWND hParent, int ControlID, std::vector<std::string>& vData)
{
	if (vData.size() > 0)
	{
		//Remove existing entries
		SendMessage(GetDlgItem(hParent , ControlID), CB_RESETCONTENT, 0, 0);
		// Now populate the graphs
		for (int i = 0; i < vData.size(); i++)
		{
			SendMessage(GetDlgItem(hParent , ControlID), CB_INSERTSTRING, i, (LPARAM)vData[i].c_str());
		}
		//Select first item
		SendMessage(GetDlgItem(hParent, ControlID), CB_SELECTSTRING, 0, (LPARAM)vData[0].c_str());
		SendMessage(GetDlgItem(hParent, ControlID), CB_SETMINVISIBLE, 5, 0);
	}
}

void UpdateDisplay(const char* szMessage)
{
	SetDlgItemText(g_hWndMainWindow, IDC_STATUS, szMessage);
}

void AddTextToDisplay(std::string sText)
{
	g_sLog += sText;
	UpdateDisplay(g_sLog.c_str());
}

BOOL CALLBACK MainDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	HRESULT hr;
	switch(Message)
	{
	case WM_INITDIALOG:
		return TRUE;

	case WM_COMMAND:
		wmId    = LOWORD(wParam);
		wmEvent = HIWORD(wParam);
		// Parse the menu selections:
		switch (wmId)
		{
		case WM_DESTROY:
			{
				PostQuitMessage(0);
				return false;
				break;
			}
		case IDC_SELECT_AUDIO_DEVICE:
			{
				//Select the input
				//Check that not "Select" was selected
				GetDlgItemText( g_hWndMainWindow, IDC_AUDIO_DEVICE, g_pszResult, 256); 
				if (strcmp(g_pszResult, "Select") != 0)
				{
					// Disable Combobox and button
					EnableWindow(GetDlgItem( g_hWndMainWindow, IDC_AUDIO_DEVICE), FALSE);
					EnableWindow(GetDlgItem( g_hWndMainWindow, IDC_SELECT_AUDIO_DEVICE), FALSE);
					g_bAudioDeviceSet = true;
				}
				break;
			}
		case IDC_START_STREAMING:
			{
				if (g_bAudioDeviceSet)
				{
					g_AudioPipeline.AudioDeviceName(std::string(g_pszResult));
					if (g_AudioPipeline.InitDirectShow())
					{
						if (g_AudioPipeline.LoadPipeLine())
						{
							g_AudioPipeline.Run();
							// Init our live555 RTSP server - do this after the audio pipeline init so that we know the media type
							//Start the live 555 RTSP server thread
							DWORD dwThreadID = 0;
							HANDLE hThread = CreateThread(0, 0, LiveRTSPServerThreadFunc, (void*)&g_AudioPipeline, 0, &dwThreadID);
						}
					}
					EnableWindow(GetDlgItem( g_hWndMainWindow, IDC_START_STREAMING), FALSE);
				}
				else
				{
					// Show message box
					MessageBox(NULL, "No audio device has been selected!", "ERROR!", MB_ICONEXCLAMATION | MB_OK);
				}

				break;
			}
		case IDC_STOP_STREAMING:
			{
				// Stop the live 555 loop
				g_WatchVariable = 1;
				// Stop the DirectShow pipeline
				g_AudioPipeline.Stop();
				// Exit
				PostQuitMessage(0);
				return false;
			}
		default:
			return DefWindowProc(hwnd, Message, wParam, lParam);
		}
		break;
	default:
		return FALSE;
	}
	return TRUE;
}

//////////////////////////////////////////////////////////////////////////
// Live Media Code
void play() {
	PollingTaskScheduler* scheduler = PollingTaskScheduler::createNew();
	env = BasicUsageEnvironment::createNew(*scheduler);

	CAudioDirectShowDeviceSource* pcmSource = CAudioDirectShowDeviceSource::createNew(*env, g_AudioPipeline.MediaHeaderInfo());
	if (pcmSource == NULL) {
		*env << "Unable to open device \""
			<< "\" as a WAV audio file source: "
			<< env->getResultMsg() << "\n";
		exit(1);
	}
	
	// Register Source with scheduler
	scheduler->registerObservableSource(pcmSource);

	g_AudioPipeline.SetLive555AudioSource(pcmSource);

	AddTextToDisplay(g_AudioPipeline.MediaHeaderInfo()->GetFormatString());

	// Get attributes of the audio source:
	unsigned char const bitsPerSample = pcmSource->bitsPerSample();
	if (bitsPerSample != 8 && bitsPerSample !=  16) {
		*env << "The input file contains " << bitsPerSample
			<< " bit-per-sample audio, which we don't handle\n";
		exit(1);
	}
	sessionState.source = pcmSource;
	unsigned const samplingFrequency = pcmSource->samplingFrequency();
	unsigned char const numChannels = pcmSource->numChannels();
	unsigned bitsPerSecond
		= samplingFrequency*bitsPerSample*numChannels;
	*env << "Audio source parameters:\n\t" << samplingFrequency << " Hz, ";
	*env << bitsPerSample << " bits-per-sample, ";
	*env << numChannels << " channels => ";
	*env << bitsPerSecond << " bits-per-second\n";

	// Add in any filter necessary to transform the data prior to streaming.
	// (This is where any audio compression would get added.)
	char* mimeType;
	unsigned char payloadFormatCode;
	if (bitsPerSample == 16) {

		// The 16-bit samples in WAV files are in little-endian order.
		// Add a filter that converts them to network (i.e., big-endian) order:
		sessionState.source = EndianSwap16::createNew(*env, pcmSource);
		if (sessionState.source == NULL) {
			*env << "Unable to create a little->bit-endian order filter from the PCM audio source: "
				<< env->getResultMsg() << "\n";
			exit(1);
		}
		mimeType = "L16";
		if (samplingFrequency == 44100 && numChannels == 2) {
			payloadFormatCode = 10; // a static RTP payload type
		} else if (samplingFrequency == 44100 && numChannels == 1) {
			payloadFormatCode = 11; // a static RTP payload type
		} else {
			payloadFormatCode = 96; // a dynamic RTP payload type
		}
		*env << "Converting to network byte order for streaming\n";
	} else { // bitsPerSample == 8
		// Don't do any transformation; send the 8-bit PCM data 'as is':
		mimeType = "L8";
		payloadFormatCode = 96; // a dynamic RTP payload type
	}

	// Create 'groupsocks' for RTP and RTCP:
	struct in_addr destinationAddress;
	destinationAddress.s_addr = chooseRandomIPv4SSMAddress(*env);
	// Note: This is a multicast address.  If you wish instead to stream
	// using unicast, then you should use the "testOnDemandRTSPServer"
	// test program - not this test program - as a model.

	const unsigned short rtpPortNum = 2222;
	const unsigned short rtcpPortNum = rtpPortNum+1;
	const unsigned char ttl = 255;

	const Port rtpPort(rtpPortNum);
	const Port rtcpPort(rtcpPortNum);

	sessionState.rtpGroupsock
		= new Groupsock(*env, destinationAddress, rtpPort, ttl);
	sessionState.rtpGroupsock->multicastSendOnly(); // we're a SSM source
	sessionState.rtcpGroupsock
		= new Groupsock(*env, destinationAddress, rtcpPort, ttl);
	sessionState.rtcpGroupsock->multicastSendOnly(); // we're a SSM source

	// Create an appropriate audio RTP sink (using "SimpleRTPSink")
	// from the RTP 'groupsock':
	sessionState.sink
		= SimpleRTPSink::createNew(*env, sessionState.rtpGroupsock,
		payloadFormatCode, samplingFrequency,
		"audio", mimeType, numChannels);

	//TOREVISE: RG TEST
	SimpleRTPSink* pSimpleSink = (SimpleRTPSink*)sessionState.sink;
	pSimpleSink->setPacketSizes(10000, 10000);

	// Create (and start) a 'RTCP instance' for this RTP sink:
	const unsigned estimatedSessionBandwidth = bitsPerSecond/1000;
	// in kbps; for RTCP b/w share
	const unsigned maxCNAMElen = 100;
	unsigned char CNAME[maxCNAMElen+1];
	gethostname((char*)CNAME, maxCNAMElen);
	CNAME[maxCNAMElen] = '\0'; // just in case
	sessionState.rtcpInstance
		= RTCPInstance::createNew(*env, sessionState.rtcpGroupsock,
		estimatedSessionBandwidth, CNAME,
		sessionState.sink, NULL /* we're a server */,
		True /* we're a SSM source*/);
	// Note: This starts RTCP running automatically

	// Create and start a RTSP server to serve this stream:
	sessionState.rtspServer = RTSPServer::createNew(*env);
	if (sessionState.rtspServer == NULL) {
		*env << "Failed to create RTSP server: " << env->getResultMsg() << "\n";
		exit(1);
	}
	ServerMediaSession* sms
		= ServerMediaSession::createNew(*env, "testStream", "DirectShowAudioSource",
		"Session streamed by \"DirectShow Live Audio Streaming Server\"", True/*SSM*/);
	sms->addSubsession(PassiveServerMediaSubsession::createNew(*sessionState.sink, sessionState.rtcpInstance));
	sessionState.rtspServer->addServerMediaSession(sms);

	char* url = sessionState.rtspServer->rtspURL(sms);
	*env << "Play this stream using the URL \"" << url << "\"\n";
	std::string sMessage = "Play this stream using the URL \"" + std::string(url) + "\r\n";
	sMessage += "The live555 library is available from http://www.live555.com\r\n";
	sMessage += "Audio Stream can be played using VLC http://www.videolan.org/vlc/ \r\n";

	AddTextToDisplay(sMessage);

	delete[] url;

	// Finally, start the streaming:
	*env << "Beginning streaming...\n";
	sessionState.sink->startPlaying(*sessionState.source, afterPlaying, NULL);
}


void afterPlaying(void* /*clientData*/) {
	*env << "...done streaming\n";

	// End by closing the media:
	Medium::close(sessionState.rtspServer);
	Medium::close(sessionState.rtcpInstance);
	Medium::close(sessionState.sink);
	delete sessionState.rtpGroupsock;
	Medium::close(sessionState.source);
	delete sessionState.rtcpGroupsock;

	// We're done:
	exit(0);
}
//////////////////////////////////////////////////////////////////////////