/** @file
===========================================================================
MODULE          : WavHeaderData

DESCRIPTIONS    : 

REVISION HISTORY: Version 1.0

COPYRIGHT       : (c)CSIR, Meraka Institute 2007 all rights reserved

RESTRICTIONS	: The information/data/code contained within this file is 
				  the property of CSIR, Meraka Institute and has been 
				  classified as PUBLIC.
===========================================================================
*/
#include "StdAfx.h"
#include "WavHeaderData.h"

WavHeaderData::WavHeaderData(void)
{
	m_samplingFrequency = 0;
	m_numChannels = 0;
	m_playTimePerSample = 0.0;
}

WavHeaderData::WavHeaderData( unsigned char audioFormat, unsigned char bitsPerSample, unsigned samplingFrequency, unsigned numChannels )
{
	m_audioFormat = audioFormat;
	m_samplingFrequency = samplingFrequency;
	m_bitsPerSample = bitsPerSample;
	m_numChannels = numChannels;
	m_playTimePerSample =  1e6/(double)m_samplingFrequency;
}

WavHeaderData::~WavHeaderData(void)
{
}

unsigned WavHeaderData::PreferredFrameSize()
{
	if (m_numChannels == 0 || m_bitsPerSample == 0)
	{
		return 0;
	}
	// Although PCM is a sample-based format, we group samples into
	// 'frames' for efficient delivery to clients.  Set up our preferred
	// frame size to be close to 20 ms, if possible, but always no greater
	// than 1400 bytes (to ensure that it will fit in a single RTP packet)
	unsigned maxSamplesPerFrame = (1400*8)/(m_numChannels*m_bitsPerSample);
	unsigned desiredSamplesPerFrame = (unsigned)(0.02*m_samplingFrequency);
	unsigned samplesPerFrame = (desiredSamplesPerFrame < maxSamplesPerFrame) ? desiredSamplesPerFrame : maxSamplesPerFrame;

	fPreferredFrameSize = (samplesPerFrame*m_numChannels*m_bitsPerSample)/8;

	return fPreferredFrameSize;
}